<?php

/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2020 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;

/**
 *
 */
class AdminCorreoArgentinoController extends ModuleAdminController
{

    protected $_module = NULL;
    const ADMIN_GLOBAL = 'Admin.Global';
    const CANCEL = 'cancel';
    const PDF = 'pdf';
    const VIEW = 'view';
    const VIEW_ORDER = 'vieworder';

    public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'correoargentino_branch_order';
        $this->className = 'CorreoArgentinoBranchOrderModel';
        $this->lang = false;

        $this->addRowAction(self::VIEW);
        $this->addRowAction(self::CANCEL);
        $this->addRowAction(self::PDF);


        $this->explicitSelect = true;
        $this->identifier = 'id_order';
        $this->list_simple_header = true;

        parent::__construct();

        $this->_select = '
		b.id_currency,
        reference,
        b.date_add,
		b.id_order AS id_pdf,
        a.branch_code,
        a.shipping_type,
        oc.tracking_number,
		CONCAT(LEFT(c.`firstname`, 1), \'. \', c.`lastname`) AS `customer`,
		osl.`name` AS `osname`,
		os.`color`,
 		country_lang.name as cname,
		IF(b.valid, 1, 0) badge_success';

        $this->_join = '
        LEFT JOIN `' . _DB_PREFIX_ . 'orders` b ON (a.`id_order` = b.`id_order`)
        LEFT JOIN `' . _DB_PREFIX_ . 'order_carrier` oc ON (a.`id_order` = oc.`id_order`)
        LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON (c.`id_customer` = b.`id_customer`)
		INNER JOIN `' . _DB_PREFIX_ . 'address` address ON address.id_address = b.id_address_delivery
		INNER JOIN `' . _DB_PREFIX_ . 'country` country ON address.id_country = country.id_country
		INNER JOIN `' . _DB_PREFIX_ . 'country_lang` country_lang ON (country.`id_country` = country_lang.`id_country` AND country_lang.`id_lang` = ' . (int)$this->context->language->id . ')
		LEFT JOIN `' . _DB_PREFIX_ . 'order_state` os ON (os.`id_order_state` = b.`current_state`)
		LEFT JOIN `' . _DB_PREFIX_ . 'order_state_lang` osl ON (os.`id_order_state` = osl.`id_order_state` AND osl.`id_lang` = ' . (int)$this->context->language->id . ')';

        $this->_where = 'and a.id_order != 0';

        $this->_orderBy = 'a.id_order';
        $this->_orderWay = 'DESC';
        $this->_use_found_rows = true;

        $this->fields_list = array(
            'id_order' => array(
                'title' => $this->trans('ID', array(), self::ADMIN_GLOBAL),
                'align' => 'text-center',
                'class' => 'fixed-width-xs',
                'filter_key' => 'b!id_order',
            ),
            'reference' => array(
                'title' => $this->trans('Reference', array(), self::ADMIN_GLOBAL),
            ),
            'tracking_number' => array(
                'title' => $this->trans('Tracking', array(), self::ADMIN_GLOBAL),
            ),
            'customer' => array(
                'title' => $this->trans('Customer', array(), self::ADMIN_GLOBAL),
                'havingFilter' => true,
            ),
            'osname' => array(
                'title' => $this->trans('Status', array(), self::ADMIN_GLOBAL),
                'type' => 'select',
                'color' => 'color',
                'filter_key' => 'os!id_order_state',
                'filter_type' => 'int',
                'order_key' => 'osname',
            ),
            'shipping_type' => array(
                'title' => $this->l('Shipping type'),
                'callback' => 'getShippingTypeTranslation',
                'callback_object' => $this
            ),
            'branch_code' => array(
                'title' => $this->l('Agency'),
                'callback' => 'getBranchName',
                'callback_object' => $this
            ),
            'date_add' => array(
                'title' => $this->trans('Date', array(), self::ADMIN_GLOBAL),
                'align' => 'text-right',
                'type' => 'datetime',
                'filter_key' => 'b!date_add',
            )
        );

        if (Tools::isSubmit('id_order')) {
            try {
                $order = new Order((int)Tools::getValue('id_order'));
                $viewOrder = Tools::getValue(self::VIEW_ORDER, self::VIEW);
                $tracking = $order->getWsShippingNumber();
                $service = CorreoArgentinoService::getInstanceWithLogin();

                if (empty($tracking)) {
                    $this->context->controller->errors[] = $this->l('You can\'t cancel an order without tracking');
                }

                if ($viewOrder == self::CANCEL) {
                    if ($service->cancel($tracking)) {
                        $order->setCurrentState(Configuration::get('PS_OS_CANCELED'));
                    } else {
                        $this->context->controller->errors[] = $this->l('You can\'t cancel this order');
                    }
                }

                if ($viewOrder == self::PDF) {
                    $label = $service->label($tracking);
                    $this->context->controller->errors[] = $label['result'];
                    CorreoArgentinoUtil::renderPDF($label);
                }

            } catch (Exception | TransportExceptionInterface $e) {
                $this->context->controller->errors[] = $e->getMessage();
                PrestaShopLogger::addLog($e->getMessage(), 3);
            }

        }
    }

    /**
     * @return string
     */
    public function getBranchName($branch_id)
    {
        $branch = CorreoArgentinoService::getBranchById($branch_id);
        if (!$branch) {
            return false;
        }
        return $branch->agency_name . ' (' . $branch->location->street_name . ' ' . $branch->location->street_number . ')';
    }

    public function getShippingTypeTranslation($type)
    {
        return $type == 'agency' ? $this->l('Branch') : $this->l('Home delivery');
    }

    public function checkOrderState($order, $state): bool
    {
        return in_array($state, $order->getCurrentOrderState()->name);
    }

    /**
     * @param null $token
     * @param $id
     * @param null $name
     * @return false|string
     * @throws SmartyException
     */
    public function displayCancelLink($token = null, $id, $name = null)
    {
        $tpl = $this->context->smarty->createTemplate(_PS_MODULE_DIR_.'/correoargentino/views/templates/admin/action_cancel.tpl');

        $order = new Order($id);
        $tracking = $order->getWsShippingNumber();

        if (!$tracking || $this->checkOrderState($order, 'Canceled')) {
            return null;
        }
        self::$cache_lang[self::CANCEL] = $this->l(ucfirst(self::CANCEL));
        $href = $this->context->link->getAdminLink('AdminCorreoArgentino', true, null, array(self::VIEW_ORDER => self::CANCEL, 'id_order' => $id));

        $tpl->assign(array(
            'href' => $href,
            'action' => self::$cache_lang[self::CANCEL],
            'id' => $id
        ));

        return $tpl->fetch();
    }


    /**
     * @param null $token
     * @param $id
     * @param null $name
     * @return false|string
     * @throws SmartyException
     */
    public function displayPdfLink($token = null, $id, $name = null)
    {
        $tpl = $this->context->smarty->createTemplate(_PS_MODULE_DIR_.'/correoargentino/views/templates/admin/action_pdf.tpl');
        $order = new Order($id);
        $tracking = $order->getWsShippingNumber();
        if (!$tracking || $this->checkOrderState($order, 'Canceled')) {
            return null;
        }
        self::$cache_lang[self::PDF] = $this->l('Label');
        $href = $this->context->link->getAdminLink('AdminCorreoArgentino', true, null, array(self::VIEW_ORDER => self::PDF, 'id_order' => $id));

        $tpl->assign(array(
            'href' => $href,
            'action' => self::$cache_lang[self::PDF],
            'id' => $id
        ));

        return $tpl->fetch();
    }

    public function displayViewLink($token = null, $id, $name = null)
    {
        $tpl = $this->createTemplate('helpers/list/list_action_view.tpl');
        self::$cache_lang[self::VIEW] = $this->l('View');
        $href = $this->context->link->getAdminLink('AdminOrders', true, null, array(self::VIEW_ORDER => '', 'id_order' => $id));

        $tpl->assign(array(
            'href' => $href,
            'action' => self::$cache_lang[self::VIEW],
            'id' => $id
        ));

        return $tpl->fetch();
    }

    public function viewAccess($disable = false)
    {
        if (version_compare(_PS_VERSION_, '1.7.6.5', '>=')) {
            return true;
        }
        return parent::viewAccess($disable);
    }

    /**
     *
     */
    public function renderView()
    {
        return parent::renderView();
    }

    /**
     *
     */
    public function initContent()
    {
        parent::initContent();
    }

    public function initToolbar()
    {
        parent::initToolbar();

        unset($this->toolbar_btn['new']);

    }

}
